//******************************************************************************
// Projeto - TESTE_ADC0831 - testa conversor AD externo de 8 bits
// Desenvolvido por Mrcio Jos Soares
//
// Arquivo - teste_adc0831.c
//
// Realiza 1 converso a cada 2 segundos (timer 0 ativo)
// e envia dado via canal serial RS232
// Comunicao: 9600, n, 8, 1
//
// Formato dos Dados:
// se RA0 = 0, envia resultado direto da converso (0 - 255)
// se RA1 = 1, envia valor convertido entre 0 e 5 VDC
//
// LED1 pisca a 1 Hz: indicao de ligado
//
// Microcontrolador utilizado: PIC16F628-04/P (no testado com verso "A")
// AD - ADC0831 com comunicao MICROWIRE (National Semicondutores)
// Compilador: CCS C Compiler - verso PCM 3.15
//
// Pinos utilizados:
// RA0 - jumper de seleo para converso
// RA1 - LED
// RB0 - Habilitao do ADC0831 (CS)
// RB1 - RX
// RB2 - TX - sada dos dados
// RB3 - Dados do ADC0831 (SO) (comunicao MicroWire)
// RB4 - Clock do ADC0831 (SK) (comunicao MicroWire)
//
// Obs:  utilizado MAX-232 para drive TTL/RS-232
//       e resistor de pull-up para pino CS
//
// I2C: A comunicao Microwire foi feita "manualmente" para
//      melhor compreenso do seu funcionamento.
//******************************************************************************

//******************************************************************************
// Arquivos includos neste mdulo
//******************************************************************************
#include "teste_adc0831.h"

//******************************************************************************
// Funes do mdulo
//******************************************************************************

//******************************************************************************
// Funo para tratar a comunicao MicroWire do ADC0831
// recebe 	- nada
// retorna 	- byte recebido do ADC0831
//******************************************************************************
char le_ADC0831(void) {

   int i;
   unsigned char conv;

   restart_WDT();					//zera WDT

   i=8;

   output_low(CLOCK);         //nvel lgico "0" no pino CLK do CI
   output_low(CS);            //seleciona CI

   output_high(CLOCK);        //pulsa pino de clock
   delay_us(10);              //por 10 us
   output_low(CLOCK);

   while(i){
      output_high(CLOCK);     //pulsa pino de clock
      delay_us(10);           //por 10 us
      output_low(CLOCK);

      shift_left(&conv, 1, input(DADOS)); //pega dado no pino
      i--;
   }

   output_high(CS);           //desabilita CI
   restart_WDT();					//zera WDT

   return(conv);              //retorna converso
}

//******************************************************************************
// Funo para tratar a interrupo para dado recebido RS-232
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_RDA
RDA_isr() {

   static char carac;

   restart_WDT();                //restaura WDT

   carac = rxreg;                //colhe dado do canal serial
   printf("%c",carac);           //envia eco de caracter recebido

   restart_WDT();                //restaura WDT
}

//******************************************************************************
// Funo para tratar a interrupo do timer0
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer0
void trata_int_t0(){

   static int contador;
   unsigned char envia;
   float conv0_5V;

   restart_WDT();							//zera WDT

   //reinicia o timer0 em 131, mas a contagem que j passou
   set_timer0(131+get_timer0());
   contador++;
   //verifica se j ocorreram 125 interrupes
   //125 Hz / 250 = 0,5 Hz = 1/0,5 = 2 segundos
   if (contador == 250){
      contador = 0;
      envia = le_ADC0831();            //le AD
      if (!convert)
         printf("Dado->%u\n\r", envia); //envia dados sem qualquer converso
      else{
         conv0_5V = envia * 0.0196;
         printf("Dado->%2.3f\n\r",conv0_5V);
      }
   }

   restart_WDT();							//zera WDT
}

//******************************************************************************
// Funo para tratar a interrupo do timer1
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer1
void trata_int_t1(){

	static int conta;
	static boolean led1;

	restart_WDT();					//zera WDT

	//reinicia timer1 com 3096, mais contagem que passou
	set_timer1(3096+get_timer1());
	conta++;
	if (conta == 2){
		conta=0;
		led1=!led1;
		output_bit(LED,led1);
		}

	restart_WDT();					//zera WDT
}

//******************************************************************************
// Funo principal
//******************************************************************************
void main(void) {

   set_tris_A(0b00000001);		//configura pinos do port A como sada,
                              //exceto RA0
   set_tris_B(0b00001010);		//configura pinos do port B como sada,
                              //exceto RB1 e RB3

   setup_comparator(nc_nc_nc_nc); //desabilita comparadores internos
	setup_wdt(WDT_2304MS);			 //habilita watch dog timer a cada 2 segundos
	restart_WDT();						//zera WDT

   //timer 0 habilitado
   //[(4MHz/4)/64]/125 = 125 Hz -> valor da frequencia escolhida
   //para obter o valor de inicializao do timer deve-se fazer (256-125)=131
   setup_timer_0(RTCC_INTERNAL | RTCC_DIV_64);
   set_timer0(131);

	//timer 1 habilitado
	// [(4MHz/4)/8]/65636 = 1.90Hz -> menor frequencia possvel
	//valor inicial do timer1 (65536-62500)=3036
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
	set_timer1(3036);
	restart_WDT();								//zera WDT

	//timer2 desabilitado
	setup_timer_2(T2_DISABLED,0,1);
	restart_WDT();								//zera WDT

   //habilita interrupes
   enable_interrupts(INT_RDA);         //habilita interrupo RS-232 (rec)
   enable_interrupts(INT_TIMER0);      //habilita interrupo do timer0
   enable_interrupts(INT_TIMER1);      //habilita interrupo do timer1
   enable_interrupts(GLOBAL);          //habilita interrupo global
   restart_WDT();							   //zera WDT

   output_high(CS);                    //mantem CI ADC0831 desabilitado

   delay_ms(1000);                     //aguarda 1 segundo para comear
   printf("P\n\r");                    //envia sinal de prontido via RS-232
   restart_WDT();	                     //zera WDT

//******************************************************************************
// Inicio do lao principal
// A leitura do AD e o envio do dado  feita via interrupo do timer 0
// O piscar do LED  controlado via interrupo do timer 1
// A recepo e retorno do caracter de eco  feita via int RDA (rec RS-232)
//******************************************************************************

   while(1){                           //faz eternamente

      if (input(JUMPER))               //apenas testa o jumper
         convert=1;                    //envia valor convertido entre 0 e 5VDC
      else
         convert=0;                    //envia converso 0 a 255 (8 bits)

      delay_ms(100);                  //l jumper a cada 100 mili segundos
      restart_WDT();

   }
}
